# We cannot obtain the length of a instruction.
# Let's create a map from insns to lengths statically.
if ARGV[0] == 'mktbl'
  puts "Generating insn.tbl..."
  ruby_path = "#{ENV['HOME']}/src/ruby"
  system("gcc -I#{ruby_path} tool/gen_insn_tbl.c -o gen_insn_tbl")
  system("./gen_insn_tbl > insn.tbl")
  puts "Done!"
  exit
end

require 'mkmf'

if !defined?(VM)
  puts "No Ruby VM. Note that Ruby 1.9 or later is required for Yajit."
  exit 1
end

INSN_TBL_FILE = 'insn.tbl'
if !File.exist?(INSN_TBL_FILE)
  puts "#{INSN_TBL_FILE}: not exists."
  exit 1
end

puts "Generating insns.inc..."
insn2len = eval(File.read('insn.tbl'))
File.open('insns.inc', 'w') do |of|
  of.puts "#define RUBY_#{RUBY_VERSION.tr('.','_')}"
  of.puts "#define RUBY_REVISION #{RUBY_REVISION}"

  of.puts "#define BIN(n) YARVINSN_##n"
  of.puts "enum {"
  VM::INSTRUCTION_NAMES.each_with_index do |insn, i|
    of.puts "#define HAVE_#{insn}"
    of.puts "  BIN(#{insn}) = #{i},"
  end
  of.puts "};"
  of.puts "#define VM_INSTRUCTION_SIZE #{VM::INSTRUCTION_NAMES.size}"

  of.puts "int insn_len_info[] = {"
  VM::INSTRUCTION_NAMES.each_with_index do |insn|
    len = insn2len[insn]
    if len
      of.puts "  #{len},"
    else
      puts "*WARNING* Your Ruby has unknown instruction: #{insn}"
      of.puts "  1,"
    end
  end
  of.puts "};"
end

have_func('mremap', 'sys/mman.h')
create_header()

CONFIG['CXXFLAGS'] += ' -fno-operator-names'
create_makefile('yajit')
