require 'test/base'
require 'yajit'

class BytecodeTest < YajitTestCase
  def assert_iseq(body)
    header = [
              "YARVInstructionSequence/SimpleDataFormat", 1, 1, 1,
              { :arg_size=>0, :local_size=>3, :stack_max=>5 },
              "<dummy>", "foo.rb", :top, [], 0, []
             ]
    if RUBY_REVISION > 0 && RUBY_REVISION < 15000
      header.insert(7, [])
    end
    iseq = Yajit::iseq_load(header + [body])
    jit = Yajit::compile_iseq(iseq)
    expected = iseq.eval
    actual = jit.call
    #puts iseq.disasm
    assert_equal(expected, actual)
  end

  def test_trivial
    body = [[:putnil], [:leave]]
    assert_iseq(body)
  end

  def test_topn
    body = [
            [:putobject, 1],
            [:putobject, 2],
            [:putobject, 3],
            [:topn, 2],
            [:opt_plus],
            [:opt_plus],
            [:opt_plus],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_adjuststack
    return if !VM::INSTRUCTION_NAMES.include?('adjuststack')
    body = [
            [:putobject, 1],
            [:putobject, 2],
            [:putobject, 3],
            [:adjuststack, 2],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_emptstack
    return if !VM::INSTRUCTION_NAMES.include?('emptstack')
    body = [
            [:putobject, 1],
            [:putobject, 2],
            [:putobject, 3],
            [:emptstack],
            [:putobject, 4],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_putcbase
    return if !VM::INSTRUCTION_NAMES.include?('putcbase')
    body = [
            [:putcbase],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_setn
    body = [
            [:putobject, 1],
            [:putobject, 2],
            [:putobject, 3],
            [:setn, 2],
            [:opt_plus],
            [:opt_plus],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_reput
    body = [
            [:putobject, 42],
            [:reput],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_swap
    body = [
            [:putobject, 42],
            [:putobject, 45],
            [:swap],
            [:opt_div],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_dupn
    body = [
            [:putobject, 42],
            [:putobject, 45],
            [:dupn, 2],
            [:opt_minus],
            [:opt_mult],
            [:opt_minus],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_dup
    body = [
            [:putobject, 42],
            [:dup],
            [:opt_mult],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_pop
    body = [
            [:putobject, 42],
            [:putobject, 45],
            [:pop],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_putself
    body = [
            [:putself],
            [:leave]
           ]
    assert_iseq(body)
  end

  def test_nop
    body = [
            [:nop],
            [:putnil],
            [:nop],
            [:leave],
            [:nop],
           ]
    assert_iseq(body)
  end

  def test_answer
    body = [
            [:answer],
            [:leave],
           ]
    assert_iseq(body)
  end

  def test_bitblt
    body = [
            [:bitblt],
            [:leave],
           ]
    assert_iseq(body)
  end

  def test_tostring
    body = [
            [:putobject, 42],
            [:tostring],
            [:concatstrings, 1],
            [:leave],
           ]
    assert_iseq(body)
  end

  def test_putstrobject
    body = [
            [:putobject, "42"],
            [:leave],
           ]
    assert_iseq(body)
  end

end
