require 'test/base'
require 'yajit'

def global_func
  l = 2
  [47, self]
end

class C
  attr_accessor :v

  def call_jit_func(jit_func)
    l = 3
    r = jit_func.call
    r + [get_iv, self, l]
  end

  def initialize
    @v = 42
  end

  def f(a,b)
    a-b
  end

  def call_f
    f(@v, 3)
  end

  def get_iv
    @v
  end

  def set_iv(a)
    @v = a
  end
end

class ClassTest < YajitTestCase
  def test_send
    c = C.new
    jit_method = Yajit.compile(c.method(:f))
    assert_equal(-3, jit_method.send(c, 1, 4))
    assert_equal(-2, jit_method.send(C.new, 2, 4))
    jit_method = Yajit.compile(C.instance_method(:f))
    assert_equal(-1, jit_method.send(c, 3, 4))
    assert_equal(0, jit_method.send(C.new, 4, 4))
  end

  def test_get_iv
    c = C.new
    jit_method = Yajit.compile(c.method(:get_iv))
    assert_equal(42, jit_method.send(c))
  end

  def test_set_iv
    c = C.new
    jit_method = Yajit.compile(c.method(:set_iv))
    jit_method.send(c, 5)
    assert_equal(5, c.get_iv)
    assert_equal(5, c.v)

    # TODO: fix?
    jit_method = Yajit.compile(c.method(:v=))
    #jit_method.send(c, 8)
    #assert_equal(8, c.v)
  end

  def test_call_other
    c = C.new
    jit_method = Yajit.compile(c.method(:call_f))
    assert_equal(39, jit_method.send(c))
  end

  def test_call_jit_func
    c = C.new
    no_jit_method = c.method(:call_jit_func)
    no_jit_func = method(:global_func)
    expected = no_jit_method.call(no_jit_func)

    jit_method = Yajit.compile(c.method(:call_jit_func))
    jit_func = Yajit.compile(method(:global_func))
    actual = jit_method.call(jit_func)

    assert_equal(expected, actual)
  end
end
