require 'test/base'
require 'yajit'

def def_method
  def defined_method
    self
  end
  defined_method
end

DEF_CLASS = %Q(
class DefinedClass
  def f
    %d
  end
end
DefinedClass.new.f
)

def singleton_method
  s = 'hoge'
  def s.to_s
    'fuga'
  end
  s.to_s
end

class DefineTest < YajitTestCase
  def test_def_method
    m = method(:def_method)
    jit = Yajit.compile(m)
    actual = jit.call
    assert_equal(self, actual)
  end

  def test_def_class
    iseq = VM::InstructionSequence.compile(DEF_CLASS % 42)
    jit = Yajit.compile_iseq(iseq)
    assert_equal(42, jit.call)
    assert_equal(42, DefinedClass.new.f)
  end

  def test_redefine_class
    iseq = VM::InstructionSequence.compile(DEF_CLASS % 42)
    jit = Yajit.compile_iseq(iseq)
    jit.call
    iseq = VM::InstructionSequence.compile(DEF_CLASS % 43)
    jit = Yajit.compile_iseq(iseq)
    assert_equal(43, jit.call)
    assert_equal(43, DefinedClass.new.f)
  end

  def test_singleton_method
    assert_jit(:singleton_method)
  end
end
